----------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;  -- defines std_logic types

entity aki80_gate is
	port (                             -- INPUT and OUTPUT declaration
		BUSAK     : in  STD_LOGIC; -- 
		RESET     : in  STD_LOGIC; -- 
		TRI_IN    : in  STD_LOGIC_VECTOR (2 downto 0); -- for MREQ, RD, WR
		TRI_OUT   : out STD_LOGIC_VECTOR (2 downto 0);
		M1        : in  STD_LOGIC; -- 
		AVR_WAIT  : in  STD_LOGIC; -- 
		A7654     : in  STD_LOGIC_VECTOR (3 downto 0); -- IO adress 7-4
		IOREQ     : in  STD_LOGIC; -- 
		CPU_WAIT  : out STD_LOGIC; -- 
		AVR_IOREQ : out STD_LOGIC; -- 
		IO_CS     : out STD_LOGIC --  external IO device select
	);
	attribute pin_assign : string;
	attribute pin_assign of busak    : signal is "1";
	attribute pin_assign of reset    : signal is "2";
	attribute pin_assign of tri_in   : signal is "44,43,42";
	attribute pin_assign of tri_out  : signal is "3,4,5";
	attribute pin_assign of m1       : signal is "38";
	attribute pin_assign of avr_wait : signal is "8";
	attribute pin_assign of a7654    : signal is "13,12,11,9";
	attribute pin_assign of ioreq    : signal is "39";
	attribute pin_assign of avr_ioreq : signal is "7";
	attribute pin_assign of cpu_wait : signal is "37";
	attribute pin_assign of io_cs    : signal is "36";
end aki80_gate;

architecture aki80_gate_arch of aki80_gate is  -- Internal signal declaration
	signal BK_NAND_RS   : STD_LOGIC; -- 
	signal AD           : STD_LOGIC; -- 
	signal AD_OR_IO     : STD_LOGIC; -- 
	signal RD_AND_WR    : STD_LOGIC; -- 
	signal INT_WAIT     : STD_LOGIC; -- 
	signal Q            : STD_LOGIC :='1'; -- Q inverse
	signal INT_CPU_WAIT : STD_LOGIC; -- 

begin
	BK_NAND_RS     <= BUSAK nand RESET;
	AD_OR_IO       <= AD or (not M1); -- M1:for Z80 Mode2 interrupt
	RD_AND_WR      <= TRI_IN(1) and TRI_IN(0); -- RD, WR
	INT_WAIT       <= AVR_WAIT or AD_OR_IO or RD_AND_WR;
	AVR_IOREQ      <= AD_OR_IO;
	INT_CPU_WAIT   <= Q and INT_WAIT;

	AD_SEL: process (A7654, IOREQ)  -- adress A7-A4, IO_CS decode
	begin
		if (A7654="1000" or	-- AVR virtual IO select
		    A7654="1001" or
		    A7654="1010" or
		    A7654="1011") then
			AD <= IOREQ;
		else
			AD <= '1';
		end if;

		if (A7654="0100") then	-- external IO device select
			IO_CS <= IOREQ;
		else
			IO_CS <= '1';
		end if;
	end process;

	OUT_Z: process (INT_CPU_WAIT)  -- output high-z (Z80 control bus. pullup to 5V)
	begin
		if(INT_CPU_WAIT='1') then CPU_WAIT <= 'Z'; else CPU_WAIT <= '0'; end if;
	end process;

	TRI_BUFF: process (BK_NAND_RS, TRI_IN) -- tristate buffer (like 74367)
	begin
		if (BK_NAND_RS='0') then
			TRI_OUT <= TRI_IN;
		else
			TRI_OUT <= "ZZZ";
		end if;
	end process;

	D_FF: process (M1, AVR_WAIT) -- D-FF (like 7474)
	begin
		if (AVR_WAIT='0') then
			Q <= '1';			-- clear
		elsif (M1'event and M1='0') then	-- M1 falling edge
			Q <= '0';
		end if;
	end process;

end aki80_gate_arch;